

#ifndef NDFA_NODE_H
#define NDFA_NODE_H

#include <exception>
#include <set>

#include "definitions.h"
#include "ndfa_accept_state.h"
#include "exceptions.h"


class NDFA_Node ;

typedef set<NDFA_Node *> NDFA_Node_Set ;
typedef NDFA_Node_Set::iterator NDFA_Node_Set_Iterator ;



class NDFA_Node
{
public:
    NDFA_Node ( int test_bit_number ) ;
    virtual ~NDFA_Node () ;


    bool Is_Deterministic ()
            { return is_deterministic ; } ;
            
    void Merge_In ( NDFA_Node * to_be_merged , bool cpr ) ;

    void Compress () ;

    bool Delete_Dead_Branches () ;

    void Print_NDFA_DAG ( bool with_pointers , FILE * fd ,
                            const char * old_tab_str = "" ) ;

    int Get_Size () ;

    NDFA_Node * Add_Transition ( bool transition_on_zero ,
                                        bool transition_on_one ) ;
    void Make_Reject_State () ;
    void Make_Accept_State ( Accept_State * accept_info ) ;

    void Make_Deterministic () ;

    NDFA_Node * DFA_Transition ( int ts ) ;

    int Get_Test_Bit_Number ()
                { return test_bit_number ; } ;

    Accept_State * Get_Accept_State () ;

    void Visit ()
                { visits ++ ; } ;
    int Get_Number_Of_Visits ()
                { return visits ; } ;




private:
    void Delete_Item_Before ( NDFA_Node_Set * set , 
                                        NDFA_Node_Set_Iterator iter ) ;
    void Check_For_Determinism () ;
    void Delete_Children () ;
    void Transform_NDFA_To_DFA ( NDFA_Node * dfa_root , int depth_remaining ) ;
    void Replace_With ( NDFA_Node * n ) ;



    Accept_State *      accept_state ;
    bool                is_accept_state ;
    bool                is_reject_state ;
    bool                is_deterministic ;
    NDFA_Node_Set       transitions [ TRANSITION_TYPES ] ;
    int                 test_bit_number ;
    int                 visits ;

    
} ;




#endif

