#ifndef STATE_MACHINE_H
#define STATE_MACHINE_H

#include <stdio.h>
#include <regex.h>


#include <set>
#include <map>
#include <list>

#include "exceptions.h"
#include "state.h"
#include "utils.h"
#include "optimisation.h"




/* The state machine class. This is able to read in a state machine
 * description, produce the VHDL it represents, and calculate the 
 * dependencies of the machine. Other state machines can be merged in. */
class State_Machine
{
public:
    State_Machine ( const char * source_file ) ;
    State_Machine () ;
    virtual ~State_Machine () ;

    void Compile_Machine ( FILE * output ) ;

    void Incorporate_Sub_Machine ( State_Machine * sub_machine ) ;

    Label_Set Depends_On () ;
    Label_Set Provides () ;

    void Finalise_SM () ;

    bool Is_Finalised ()
            { return is_finalised ; } ;

    State * Get_State_For_Name ( const char * sub_name ) ;

    int Get_Width_Of_State_Number ()
            { return Get_Number_Of_Bits_Needed_For 
                                ( states . size () - 1 ) ; } ;

    const char * Get_Name ()
            { return name ; } ;
private:
    void Calculate_Abs_State_Numbers () ;


    State_List      states ;
    State_Map       definitions ;
    bool            is_finalised ;
    const char *    name ;
} ;

#endif

